@@ -3,14 +3,9 @@ package App::paperback;
 use v5.10;
 use strict;
 # use warnings;
-our $VERSION = "v1.00";
+our $VERSION = "v1.03.01";
 
-use Exporter;
-our @ISA    = qw(Exporter);
-our @EXPORT = qw(openInputFile openOutputFile newPageInOutputFile
-	copyPageFromInputToOutput closeInputFile closeOutputFile);
-
-my ($GformNr, $GinFile, $GpageObjNr, $Groot, $Gpos, $GobjNr, $Gstream, $GoWid, $GoHei);
+my ($GinFile, $GpageObjNr, $Groot, $Gpos, $GobjNr, $Gstream, $GoWid, $GoHei);
 my (@Gkids, @Gcounts, @GformBox, @Gobject, @Gparents, @Gto_be_created);
 my (%Gold, %Gresources, %GpageXObject, %GoldObject);
 
@@ -110,6 +105,8 @@ ANSI normalised (US) page sizes:
   Before that, input pages will be reordered and reoriented so as to 
   produce a final PDF fit for duplex 'long-edge-flip' printing.
 
+For further details, please try 'perldoc paperback'.
+
 ${sayVersion}
 END_MESSAGE
 
@@ -119,28 +116,27 @@ END_MESSAGE
     if $input =~ "^-h\$" or $input =~ "^--help\$";
   do {print STDERR "${sayVersion}\n" and exit}
     if $input =~ "^-v\$" or $input =~ "^--version\$";
-  die "[!] File '$input' can't be found or read.\n"
+  die "[!] File '$input' can't be found or sysread.\n"
     unless -r $input;
   ($num_pag_input, $pgSizeInput) = openInputFile($input);
   die "[!] File '$input' is not a valid v1.4 PDF file.\n"
     if $num_pag_input == 0;
 
-  my ($pagesPerSheet, @x, @y);
+  my ($pgPerOutputPage, @x, @y);
   for ($pgSizeInput) {
-       if ($_ eq "A6") { $pagesPerSheet = 4; @x = @X_A6_ON_A4; @y = @Y_A6_ON_A4; }
-    elsif ($_ eq "A5") { $pagesPerSheet = 2; @x = @X_A5_ON_A4; @y = @Y_A5_ON_A4; }
-    elsif ($_ eq "QT") { $pagesPerSheet = 4; @x = @X_QT_ON_LT; @y = @Y_QT_ON_LT; }
-    elsif ($_ eq "QG") { $pagesPerSheet = 4; @x = @X_QG_ON_LG; @y = @Y_QG_ON_LG; }
-    elsif ($_ eq "HT") { $pagesPerSheet = 2; @x = @X_HT_ON_LT; @y = @Y_HT_ON_LT; }
-    elsif ($_ eq "HG") { $pagesPerSheet = 2; @x = @X_HG_ON_LG; @y = @Y_HG_ON_LG; }
+       if ($_ eq "A6") { $pgPerOutputPage = 4; @x = @X_A6_ON_A4; @y = @Y_A6_ON_A4; }
+    elsif ($_ eq "A5") { $pgPerOutputPage = 2; @x = @X_A5_ON_A4; @y = @Y_A5_ON_A4; }
+    elsif ($_ eq "QT") { $pgPerOutputPage = 4; @x = @X_QT_ON_LT; @y = @Y_QT_ON_LT; }
+    elsif ($_ eq "QG") { $pgPerOutputPage = 4; @x = @X_QG_ON_LG; @y = @Y_QG_ON_LG; }
+    elsif ($_ eq "HT") { $pgPerOutputPage = 2; @x = @X_HT_ON_LT; @y = @Y_HT_ON_LT; }
+    elsif ($_ eq "HG") { $pgPerOutputPage = 2; @x = @X_HG_ON_LG; @y = @Y_HG_ON_LG; }
     else {die "[!] Bad page size ($pgSizeInput). See 'paperback -h' to learn more.\n"}
   }
 
   my ($name) = $input =~ /(.+)\.[^.]+$/;
   openOutputFile("${name}-paperback.pdf");
-  my $num_pliegos = $num_pag_input >> 4;
   my ($rot_extra, @p);
-  if ($pagesPerSheet == 4) {
+  if ($pgPerOutputPage == 4) {
   	$rot_extra = 0;
   	   if ($num_pag_input >= 13) { @p = @P_4UP_13PLUS; } 
   	elsif ($num_pag_input >= 9 ) { @p = @P_4UP_9PLUS;  } 
@@ -153,11 +149,12 @@ END_MESSAGE
   	elsif ($num_pag_input >= 5 ) { @p = @P_2UP_5PLUS;  } 
   	else                         { @p = @P_2UP_1PLUS;  }
   }
+  my $lastSignature = $num_pag_input >> 4;
   my ($rotation, $target_page);
-  for (0..$num_pliegos) {
+  for (my $thisSignature = 0; $thisSignature <= $lastSignature; ++$thisSignature) {
     for (0..15) {
-      newPageInOutputFile() if $_ % $pagesPerSheet == 0;
-      $target_page = $p[$_] + ($numPagImposed >> 4) * 16;
+      newPageInOutputFile() if $_ % $pgPerOutputPage == 0;
+      $target_page = $p[$_] + 16 * $thisSignature;
       next if $target_page > $num_pag_input;
 
       $rotation = $_ % 4 > 1 ? $rot_extra + 180 : $rot_extra;
@@ -198,8 +195,10 @@ sub copyPageFromInputToOutput {
   my $y          = $param->{'y'}      or 0;
   my $rotate     = $param->{'rotate'} or 0;
 
-  ++$GformNr;
-  my $name = "Fm${GformNr}";
+  state $formNr; # Este uso de "state" requiere v5.10 (que salió en 2007)
+  ++$formNr;
+
+  my $name = "Fm${formNr}";
   my $refNr = getPage( $pagenumber );
   die "[!] Page ${pagenumber} in ${GinFile} can't be used. Concatenate streams!" 
   	if !defined $refNr;
@@ -483,6 +482,7 @@ sub getObject {
 
   sysseek $IN_FILE, $offs, 0;
   sysread $IN_FILE, $buf, $size;
+
   return $buf;
 }
 
@@ -491,7 +491,7 @@ sub getObject {
 sub writeToBeCreated {
 ##########################################################
   my $elObje = $_[0];
-  my ( $out_line, $part, $strPos);
+  my ($out_line, $part, $strPos);
 
   for (@Gto_be_created) {
     my $old_one = $_->[0];
@@ -534,33 +534,27 @@ sub getInputPageDimensions {
     . int($GformBox[3] / 72 * 25.4) . " mm";
 
   if ($multi > 119_780 and $multi < 121_780) { # US 1/4 letter: 120780
-    $GoWid  = $DW;
-    $GoHei = $DH;
+    $GoWid = $DW; $GoHei = $DH;
     return "QT";
   }
   if ($multi > 123_443 and $multi < 125_443) { # ISO A6: 124443
-    $GoWid  = $AW;
-    $GoHei = $AH;
+    $GoWid = $AW; $GoHei = $AH;
     return "A6";
   }
   if ($multi > 152_720 and $multi < 154_720) { # US 1/4 legal: 153720
-    $GoWid  = $GW;
-    $GoHei = $GH;
+    $GoWid = $GW; $GoHei = $GH;
     return "QG";
   }
   if ($multi > 241_352 and $multi < 243_352) { # US 1/2 Letter ("statement"): 242352
-    $GoWid  = $DW;
-    $GoHei = $DH;
+    $GoWid = $DW; $GoHei = $DH;
     return "HT";
   } 
   if ($multi > 248_305 and $multi < 250_305) { # ISO A5: 249305
-    $GoWid  = $AW;
-    $GoHei = $AH;
+    $GoWid = $AW; $GoHei = $AH;
     return "A5";
   }
   if ($multi > 307_448 and $multi < 309_448) { # US 1/2 legal: 308448
-    $GoWid  = $GW;
-    $GoHei = $GH;
+    $GoWid = $GW; $GoHei = $GH;
     return "HG";
   }
   if ($multi > 483_704 and $multi < 485_704) { # US letter: 484704
